@@ -48,6 +48,7 @@
 #include "advapi32_misc.h"
 
 #include "wine/exception.h"
+#include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(service);
 
@@ -83,6 +84,18 @@ typedef struct dispatcher_data_t
     HANDLE pipe;
 } dispatcher_data;
 
+typedef struct notify_data_t {
+    SC_HANDLE service;
+    SC_RPC_NOTIFY_PARAMS params;
+    SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2 cparams;
+    SC_NOTIFY_RPC_HANDLE notify_handle;
+    SERVICE_NOTIFYW *notify_buffer;
+    HANDLE calling_thread, ready_evt;
+    struct list entry;
+} notify_data;
+
+static struct list notify_list = LIST_INIT(notify_list);
+
 static CRITICAL_SECTION service_cs;
 static CRITICAL_SECTION_DEBUG service_cs_debug =
 {
@@ -2633,37 +2646,130 @@ BOOL WINAPI EnumDependentServicesW( SC_HANDLE hService, DWORD dwServiceState,
     return TRUE;
 }
 
+static DWORD WINAPI notify_thread(void *user)
+{
+    DWORD err;
+    notify_data *data = user;
+    SC_RPC_NOTIFY_PARAMS_LIST *list = NULL;
+    SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2 *cparams;
+    BOOL dummy;
+
+    __TRY
+    {
+        /* GetNotifyResults blocks until there is an event */
+        err = svcctl_GetNotifyResults(data->notify_handle, &list);
+    }
+    __EXCEPT(rpc_filter)
+    {
+        err = map_exception_code(GetExceptionCode());
+    }
+    __ENDTRY
+
+    EnterCriticalSection( &service_cs );
+
+    list_remove(&data->entry);
+
+    LeaveCriticalSection( &service_cs );
+
+    if (err == ERROR_SUCCESS && list)
+    {
+        cparams = list->NotifyParamsArray[0].u.params;
+
+        data->notify_buffer->dwNotificationStatus = cparams->dwNotificationStatus;
+        memcpy(&data->notify_buffer->ServiceStatus, &cparams->ServiceStatus,
+                sizeof(SERVICE_STATUS_PROCESS));
+        data->notify_buffer->dwNotificationTriggered = cparams->dwNotificationTriggered;
+        data->notify_buffer->pszServiceNames = NULL;
+
+        QueueUserAPC((PAPCFUNC)data->notify_buffer->pfnNotifyCallback,
+                data->calling_thread, (ULONG_PTR)data->notify_buffer);
+
+        HeapFree(GetProcessHeap(), 0, list);
+    }
+    else
+        WARN("GetNotifyResults server call failed: %u\n", err);
+
+
+    __TRY
+    {
+        err = svcctl_CloseNotifyHandle(&data->notify_handle, &dummy);
+    }
+    __EXCEPT(rpc_filter)
+    {
+        err = map_exception_code(GetExceptionCode());
+    }
+    __ENDTRY
+
+    if (err != ERROR_SUCCESS)
+        WARN("CloseNotifyHandle server call failed: %u\n", err);
+
+    CloseHandle(data->calling_thread);
+    HeapFree(GetProcessHeap(), 0, data);
+
+    return 0;
+}
+
 /******************************************************************************
  * NotifyServiceStatusChangeW [ADVAPI32.@]
  */
 DWORD WINAPI NotifyServiceStatusChangeW(SC_HANDLE hService, DWORD dwNotifyMask,
         SERVICE_NOTIFYW *pNotifyBuffer)
 {
-    DWORD dummy;
-    BOOL ret;
-    SERVICE_STATUS_PROCESS st;
-    static int once;
+    DWORD err;
+    BOOL b_dummy = FALSE;
+    GUID g_dummy = {0};
+    notify_data *data;
 
-    if (!once++) FIXME("%p 0x%x %p - semi-stub\n", hService, dwNotifyMask, pNotifyBuffer);
+    TRACE("%p 0x%x %p\n", hService, dwNotifyMask, pNotifyBuffer);
 
-    ret = QueryServiceStatusEx(hService, SC_STATUS_PROCESS_INFO, (void*)&st, sizeof(st), &dummy);
-    if (ret)
+    data = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*data));
+    if (!data)
+        return ERROR_NOT_ENOUGH_MEMORY;
+
+    data->service = hService;
+    data->notify_buffer = pNotifyBuffer;
+    if (!DuplicateHandle(GetCurrentProcess(), GetCurrentThread(),
+            GetCurrentProcess(), &data->calling_thread, 0, FALSE,
+            DUPLICATE_SAME_ACCESS))
     {
-        /* dwNotifyMask is a set of bitflags in same order as SERVICE_ statuses */
-        if (dwNotifyMask & (1 << (st.dwCurrentState - SERVICE_STOPPED)))
-        {
-            pNotifyBuffer->dwNotificationStatus = ERROR_SUCCESS;
-            memcpy(&pNotifyBuffer->ServiceStatus, &st, sizeof(pNotifyBuffer->ServiceStatus));
-            pNotifyBuffer->dwNotificationTriggered = 1 << (st.dwCurrentState - SERVICE_STOPPED);
-            pNotifyBuffer->pszServiceNames = NULL;
-            TRACE("Queueing notification: 0x%x\n", pNotifyBuffer->dwNotificationTriggered);
-            QueueUserAPC((PAPCFUNC)pNotifyBuffer->pfnNotifyCallback,
-                    GetCurrentThread(), (ULONG_PTR)pNotifyBuffer);
-        }
+        ERR("DuplicateHandle failed: %u\n", GetLastError());
+        HeapFree(GetProcessHeap(), 0, data);
+        return ERROR_NOT_ENOUGH_MEMORY;
     }
 
-    /* TODO: If the service is not currently in a matching state, we should
-     * tell `services` to monitor it. */
+    data->params.dwInfoLevel = 2;
+    data->params.u.params = &data->cparams;
+
+    data->cparams.dwNotifyMask = dwNotifyMask;
+
+    EnterCriticalSection( &service_cs );
+
+    __TRY
+    {
+        err = svcctl_NotifyServiceStatusChange(hService, data->params,
+                &g_dummy, &g_dummy, &b_dummy, &data->notify_handle);
+    }
+    __EXCEPT(rpc_filter)
+    {
+        err = map_exception_code(GetExceptionCode());
+    }
+    __ENDTRY
+
+    if (err != ERROR_SUCCESS)
+    {
+        WARN("NotifyServiceStatusChange server call failed: %u\n", err);
+        LeaveCriticalSection( &service_cs );
+        CloseHandle(data->calling_thread);
+        CloseHandle(data->ready_evt);
+        HeapFree(GetProcessHeap(), 0, data);
+        return err;
+    }
+
+    CloseHandle(CreateThread(NULL, 0, &notify_thread, data, 0, NULL));
+
+    list_add_tail(&notify_list, &data->entry);
+
+    LeaveCriticalSection( &service_cs );
 
     return ERROR_SUCCESS;
 }
