@@ -42,6 +42,8 @@
 
 #define SEL_NOTIFY_TEST_ID  100
 
+static BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
+
 static struct msg_sequence *sequences[NUM_MSG_SEQUENCES];
 
 static HWND parent_wnd;
@@ -598,8 +600,6 @@ static HWND create_parent_window(void)
 {
     HWND hwnd;
 
-    InitCommonControls();
-
     /* flush message sequences, so we can check the new sequence by the end of function */
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
 
@@ -621,15 +621,14 @@ static LRESULT WINAPI monthcal_subclass_proc(HWND hwnd, UINT message, WPARAM wPa
 {
     WNDPROC oldproc = (WNDPROC)GetWindowLongPtrA(hwnd, GWLP_USERDATA);
     static LONG defwndproc_counter = 0;
+    struct message msg = { 0 };
     LRESULT ret;
-    struct message msg;
 
     msg.message = message;
     msg.flags = sent|wparam|lparam;
     if (defwndproc_counter) msg.flags |= defwinproc;
     msg.wParam = wParam;
     msg.lParam = lParam;
-    msg.id = 0;
     add_message(sequences, MONTHCAL_SEQ_INDEX, &msg);
 
     /* some debug output for style changing */
@@ -1217,7 +1216,7 @@ if (0)
             } else {
                 title_index++;
 
-                if (sizeof(title_hits) / sizeof(title_hits[0]) <= title_index)
+                if (ARRAY_SIZE(title_hits) <= title_index)
                     break;
 
                 todo_wine_if(title_hits[title_index].todo)
@@ -1242,8 +1241,7 @@ if (0)
 
     todo_wine ok(month_count + year_count >= 1, "Not enough month and year items\n");
 
-    ok(r.right <= x && title_index + 1 == sizeof(title_hits) / sizeof(title_hits[0]),
-       "Wrong title layout\n");
+    ok(r.right <= x && title_index + 1 == ARRAY_SIZE(title_hits), "Wrong title layout\n");
 
     DestroyWindow(hwnd);
 }
@@ -2017,7 +2015,7 @@ static void test_sel_notify(void)
     };
     int i;
 
-    for(i = 0; i < sizeof styles / sizeof styles[0]; i++)
+    for(i = 0; i < ARRAY_SIZE(styles); i++)
     {
         hwnd = create_monthcal_control(styles[i].val);
         SetWindowLongPtrA(hwnd, GWLP_ID, SEL_NOTIFY_TEST_ID);
@@ -2040,22 +2038,23 @@ static void test_sel_notify(void)
     }
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+    X(InitCommonControlsEx);
+#undef X
+}
+
 START_TEST(monthcal)
 {
-    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
     INITCOMMONCONTROLSEX iccex;
-    HMODULE hComctl32;
-
     ULONG_PTR ctx_cookie;
     HANDLE hCtx;
 
-    hComctl32 = GetModuleHandleA("comctl32.dll");
-    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
-    if (!pInitCommonControlsEx)
-    {
-        win_skip("InitCommonControlsEx() is missing. Skipping the tests\n");
-        return;
-    }
+    init_functions();
+
     iccex.dwSize = sizeof(iccex);
     iccex.dwICC  = ICC_DATE_CLASSES;
     pInitCommonControlsEx(&iccex);
