@@ -1,5 +1,5 @@
 /*
- * Copyright 2016 Matteo Bruni for CodeWeavers
+ * Copyright 2016 Andrey Gusev
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,8 @@
 
 #include "config.h"
 #include "wine/port.h"
-#include "d3dx11.h"
+#include "d3d10_1.h"
+#include "d3dx10.h"
 #include "d3dcompiler.h"
 
 #include "wine/debug.h"
@@ -28,7 +29,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3dx);
 
 struct asyncdataloader
 {
-    ID3DX11DataLoader ID3DX11DataLoader_iface;
+    ID3DX10DataLoader ID3DX10DataLoader_iface;
 
     union
     {
@@ -46,20 +47,20 @@ struct asyncdataloader
     SIZE_T size;
 };
 
-static inline struct asyncdataloader *impl_from_ID3DX11DataLoader(ID3DX11DataLoader *iface)
+static inline struct asyncdataloader *impl_from_ID3DX10DataLoader(ID3DX10DataLoader *iface)
 {
-    return CONTAINING_RECORD(iface, struct asyncdataloader, ID3DX11DataLoader_iface);
+    return CONTAINING_RECORD(iface, struct asyncdataloader, ID3DX10DataLoader_iface);
 }
 
-static HRESULT WINAPI memorydataloader_Load(ID3DX11DataLoader *iface)
+static HRESULT WINAPI memorydataloader_Load(ID3DX10DataLoader *iface)
 {
     TRACE("iface %p.\n", iface);
     return S_OK;
 }
 
-static HRESULT WINAPI memorydataloader_Decompress(ID3DX11DataLoader *iface, void **data, SIZE_T *size)
+static HRESULT WINAPI memorydataloader_Decompress(ID3DX10DataLoader *iface, void **data, SIZE_T *size)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p, data %p, size %p.\n", iface, data, size);
 
@@ -69,9 +70,9 @@ static HRESULT WINAPI memorydataloader_Decompress(ID3DX11DataLoader *iface, void
     return S_OK;
 }
 
-static HRESULT WINAPI memorydataloader_Destroy(ID3DX11DataLoader *iface)
+static HRESULT WINAPI memorydataloader_Destroy(ID3DX10DataLoader *iface)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p.\n", iface);
 
@@ -79,16 +80,16 @@ static HRESULT WINAPI memorydataloader_Destroy(ID3DX11DataLoader *iface)
     return S_OK;
 }
 
-static const ID3DX11DataLoaderVtbl memorydataloadervtbl =
+static const ID3DX10DataLoaderVtbl memorydataloadervtbl =
 {
     memorydataloader_Load,
     memorydataloader_Decompress,
     memorydataloader_Destroy
 };
 
-static HRESULT WINAPI filedataloader_Load(ID3DX11DataLoader *iface)
+static HRESULT WINAPI filedataloader_Load(ID3DX10DataLoader *iface)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
     DWORD size, read_len;
     HANDLE file;
     void *data;
@@ -100,7 +101,7 @@ static HRESULT WINAPI filedataloader_Load(ID3DX11DataLoader *iface)
     file = CreateFileW(loader->u.file.path, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
             NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
     if (file == INVALID_HANDLE_VALUE)
-        return D3D11_ERROR_FILE_NOT_FOUND;
+        return D3D10_ERROR_FILE_NOT_FOUND;
 
     size = GetFileSize(file, NULL);
     data = HeapAlloc(GetProcessHeap(), 0, size);
@@ -114,7 +115,7 @@ static HRESULT WINAPI filedataloader_Load(ID3DX11DataLoader *iface)
     CloseHandle(file);
     if (!ret)
     {
-        ERR("Failed to read file contents.\n");
+        WARN("Failed to read file contents.\n");
         HeapFree(GetProcessHeap(), 0, data);
         return E_FAIL;
     }
@@ -126,9 +127,9 @@ static HRESULT WINAPI filedataloader_Load(ID3DX11DataLoader *iface)
     return S_OK;
 }
 
-static HRESULT WINAPI filedataloader_Decompress(ID3DX11DataLoader *iface, void **data, SIZE_T *size)
+static HRESULT WINAPI filedataloader_Decompress(ID3DX10DataLoader *iface, void **data, SIZE_T *size)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p, data %p, size %p.\n", iface, data, size);
 
@@ -141,9 +142,9 @@ static HRESULT WINAPI filedataloader_Decompress(ID3DX11DataLoader *iface, void *
     return S_OK;
 }
 
-static HRESULT WINAPI filedataloader_Destroy(ID3DX11DataLoader *iface)
+static HRESULT WINAPI filedataloader_Destroy(ID3DX10DataLoader *iface)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p.\n", iface);
 
@@ -154,16 +155,16 @@ static HRESULT WINAPI filedataloader_Destroy(ID3DX11DataLoader *iface)
     return S_OK;
 }
 
-static const ID3DX11DataLoaderVtbl filedataloadervtbl =
+static const ID3DX10DataLoaderVtbl filedataloadervtbl =
 {
     filedataloader_Load,
     filedataloader_Decompress,
     filedataloader_Destroy
 };
 
-static HRESULT WINAPI resourcedataloader_Load(ID3DX11DataLoader *iface)
+static HRESULT WINAPI resourcedataloader_Load(ID3DX10DataLoader *iface)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
     HGLOBAL hglobal;
 
     TRACE("iface %p.\n", iface);
@@ -174,7 +175,7 @@ static HRESULT WINAPI resourcedataloader_Load(ID3DX11DataLoader *iface)
     hglobal = LoadResource(loader->u.resource.module, loader->u.resource.rsrc);
     if (!hglobal)
     {
-        ERR("Failed to load resource.\n");
+        WARN("Failed to load resource.\n");
         return E_FAIL;
     }
 
@@ -184,9 +185,9 @@ static HRESULT WINAPI resourcedataloader_Load(ID3DX11DataLoader *iface)
     return S_OK;
 }
 
-static HRESULT WINAPI resourcedataloader_Decompress(ID3DX11DataLoader *iface, void **data, SIZE_T *size)
+static HRESULT WINAPI resourcedataloader_Decompress(ID3DX10DataLoader *iface, void **data, SIZE_T *size)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p, data %p, size %p.\n", iface, data, size);
 
@@ -199,9 +200,9 @@ static HRESULT WINAPI resourcedataloader_Decompress(ID3DX11DataLoader *iface, vo
     return S_OK;
 }
 
-static HRESULT WINAPI resourcedataloader_Destroy(ID3DX11DataLoader *iface)
+static HRESULT WINAPI resourcedataloader_Destroy(ID3DX10DataLoader *iface)
 {
-    struct asyncdataloader *loader = impl_from_ID3DX11DataLoader(iface);
+    struct asyncdataloader *loader = impl_from_ID3DX10DataLoader(iface);
 
     TRACE("iface %p.\n", iface);
 
@@ -210,16 +211,16 @@ static HRESULT WINAPI resourcedataloader_Destroy(ID3DX11DataLoader *iface)
     return S_OK;
 }
 
-static const ID3DX11DataLoaderVtbl resourcedataloadervtbl =
+static const ID3DX10DataLoaderVtbl resourcedataloadervtbl =
 {
     resourcedataloader_Load,
     resourcedataloader_Decompress,
     resourcedataloader_Destroy
 };
 
-HRESULT WINAPI D3DX11CompileFromMemory(const char *data, SIZE_T data_size, const char *filename,
+HRESULT WINAPI D3DX10CompileFromMemory(const char *data, SIZE_T data_size, const char *filename,
         const D3D10_SHADER_MACRO *defines, ID3D10Include *include, const char *entry_point,
-        const char *target, UINT sflags, UINT eflags, ID3DX11ThreadPump *pump, ID3D10Blob **shader,
+        const char *target, UINT sflags, UINT eflags, ID3DX10ThreadPump *pump, ID3D10Blob **shader,
         ID3D10Blob **error_messages, HRESULT *hresult)
 {
     TRACE("data %s, data_size %lu, filename %s, defines %p, include %p, entry_point %s, target %s, "
@@ -229,37 +230,37 @@ HRESULT WINAPI D3DX11CompileFromMemory(const char *data, SIZE_T data_size, const
             error_messages, hresult);
 
     if (pump)
-        FIXME("Unimplemented ID3DX11ThreadPump handling.\n");
+        FIXME("Unimplemented ID3DX10ThreadPump handling.\n");
 
     return D3DCompile(data, data_size, filename, defines, include, entry_point, target,
             sflags, eflags, shader, error_messages);
 }
 
-HRESULT WINAPI D3DX11CompileFromFileA(const char *filename, const D3D10_SHADER_MACRO *defines,
-        ID3D10Include *include, const char *entry_point, const char *target, UINT sflags, UINT eflags,
-        ID3DX11ThreadPump *pump, ID3D10Blob **shader, ID3D10Blob **error_messages, HRESULT *hresult)
+HRESULT WINAPI D3DX10CreateEffectPoolFromFileA(const char *filename, const D3D10_SHADER_MACRO *defines,
+        ID3D10Include *include, const char *profile, UINT hlslflags, UINT fxflags, ID3D10Device *device,
+        ID3DX10ThreadPump *pump, ID3D10EffectPool **effectpool, ID3D10Blob **errors, HRESULT *hresult)
 {
-    FIXME("filename %s, defines %p, include %p, entry_point %s, target %s, sflags %#x, "
-            "eflags %#x, pump %p, shader %p, error_messages %p, hresult %p stub.\n",
-            debugstr_a(filename), defines, include, debugstr_a(entry_point), debugstr_a(target),
-            sflags, eflags, pump, shader, error_messages, hresult);
+    FIXME("filename %s, defines %p, include %p, profile %s, hlslflags %#x, fxflags %#x, device %p, "
+            "pump %p, effectpool %p, errors %p, hresult %p, stub!\n",
+            debugstr_a(filename), defines, include, debugstr_a(profile), hlslflags, fxflags, device,
+            pump, effectpool, errors, hresult);
 
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI D3DX11CompileFromFileW(const WCHAR *filename, const D3D10_SHADER_MACRO *defines,
-        ID3D10Include *include, const char *entry_point, const char *target, UINT sflags, UINT eflags,
-        ID3DX11ThreadPump *pump, ID3D10Blob **shader, ID3D10Blob **error_messages, HRESULT *hresult)
+HRESULT WINAPI D3DX10CreateEffectPoolFromFileW(const WCHAR *filename, const D3D10_SHADER_MACRO *defines,
+        ID3D10Include *include, const char *profile, UINT hlslflags, UINT fxflags, ID3D10Device *device,
+        ID3DX10ThreadPump *pump, ID3D10EffectPool **effectpool, ID3D10Blob **errors, HRESULT *hresult)
 {
-    FIXME("filename %s, defines %p, include %p, entry_point %s, target %s, sflags %#x, "
-            "eflags %#x, pump %p, shader %p, error_messages %p, hresult %p stub.\n",
-            debugstr_w(filename), defines, include, debugstr_a(entry_point), debugstr_a(target),
-            sflags, eflags, pump, shader, error_messages, hresult);
+    FIXME("filename %s, defines %p, include %p, profile %s, hlslflags %#x, fxflags %#x, device %p, "
+            "pump %p, effectpool %p, errors %p, hresult %p, stub!\n",
+            debugstr_w(filename), defines, include, debugstr_a(profile), hlslflags, fxflags, device,
+            pump, effectpool, errors, hresult);
 
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI D3DX11CreateAsyncMemoryLoader(const void *data, SIZE_T data_size, ID3DX11DataLoader **loader)
+HRESULT WINAPI D3DX10CreateAsyncMemoryLoader(const void *data, SIZE_T data_size, ID3DX10DataLoader **loader)
 {
     struct asyncdataloader *object;
 
@@ -272,16 +273,16 @@ HRESULT WINAPI D3DX11CreateAsyncMemoryLoader(const void *data, SIZE_T data_size,
     if (!object)
         return E_OUTOFMEMORY;
 
-    object->ID3DX11DataLoader_iface.lpVtbl = &memorydataloadervtbl;
+    object->ID3DX10DataLoader_iface.lpVtbl = &memorydataloadervtbl;
     object->data = (void *)data;
     object->size = data_size;
 
-    *loader = &object->ID3DX11DataLoader_iface;
+    *loader = &object->ID3DX10DataLoader_iface;
 
     return S_OK;
 }
 
-HRESULT WINAPI D3DX11CreateAsyncFileLoaderA(const char *filename, ID3DX11DataLoader **loader)
+HRESULT WINAPI D3DX10CreateAsyncFileLoaderA(const char *filename, ID3DX10DataLoader **loader)
 {
     WCHAR *filename_w;
     HRESULT hr;
@@ -296,14 +297,14 @@ HRESULT WINAPI D3DX11CreateAsyncFileLoaderA(const char *filename, ID3DX11DataLoa
     filename_w = HeapAlloc(GetProcessHeap(), 0, len * sizeof(*filename_w));
     MultiByteToWideChar(CP_ACP, 0, filename, -1, filename_w, len);
 
-    hr = D3DX11CreateAsyncFileLoaderW(filename_w, loader);
+    hr = D3DX10CreateAsyncFileLoaderW(filename_w, loader);
 
     HeapFree(GetProcessHeap(), 0, filename_w);
 
     return hr;
 }
 
-HRESULT WINAPI D3DX11CreateAsyncFileLoaderW(const WCHAR *filename, ID3DX11DataLoader **loader)
+HRESULT WINAPI D3DX10CreateAsyncFileLoaderW(const WCHAR *filename, ID3DX10DataLoader **loader)
 {
     struct asyncdataloader *object;
 
@@ -316,7 +317,7 @@ HRESULT WINAPI D3DX11CreateAsyncFileLoaderW(const WCHAR *filename, ID3DX11DataLo
     if (!object)
         return E_OUTOFMEMORY;
 
-    object->ID3DX11DataLoader_iface.lpVtbl = &filedataloadervtbl;
+    object->ID3DX10DataLoader_iface.lpVtbl = &filedataloadervtbl;
     object->u.file.path = HeapAlloc(GetProcessHeap(), 0, (strlenW(filename) + 1) * sizeof(WCHAR));
     if (!object->u.file.path)
     {
@@ -327,12 +328,12 @@ HRESULT WINAPI D3DX11CreateAsyncFileLoaderW(const WCHAR *filename, ID3DX11DataLo
     object->data = NULL;
     object->size = 0;
 
-    *loader = &object->ID3DX11DataLoader_iface;
+    *loader = &object->ID3DX10DataLoader_iface;
 
     return S_OK;
 }
 
-HRESULT WINAPI D3DX11CreateAsyncResourceLoaderA(HMODULE module, const char *resource, ID3DX11DataLoader **loader)
+HRESULT WINAPI D3DX10CreateAsyncResourceLoaderA(HMODULE module, const char *resource, ID3DX10DataLoader **loader)
 {
     struct asyncdataloader *object;
     HRSRC rsrc;
@@ -348,23 +349,23 @@ HRESULT WINAPI D3DX11CreateAsyncResourceLoaderA(HMODULE module, const char *reso
 
     if (!(rsrc = FindResourceA(module, resource, (const char *)RT_RCDATA)))
     {
-        ERR("Failed to find resource.\n");
+        WARN("Failed to find resource.\n");
         HeapFree(GetProcessHeap(), 0, object);
-        return D3DX11_ERR_INVALID_DATA;
+        return D3DX10_ERR_INVALID_DATA;
     }
 
-    object->ID3DX11DataLoader_iface.lpVtbl = &resourcedataloadervtbl;
+    object->ID3DX10DataLoader_iface.lpVtbl = &resourcedataloadervtbl;
     object->u.resource.module = module;
     object->u.resource.rsrc = rsrc;
     object->data = NULL;
     object->size = 0;
 
-    *loader = &object->ID3DX11DataLoader_iface;
+    *loader = &object->ID3DX10DataLoader_iface;
 
     return S_OK;
 }
 
-HRESULT WINAPI D3DX11CreateAsyncResourceLoaderW(HMODULE module, const WCHAR *resource, ID3DX11DataLoader **loader)
+HRESULT WINAPI D3DX10CreateAsyncResourceLoaderW(HMODULE module, const WCHAR *resource, ID3DX10DataLoader **loader)
 {
     struct asyncdataloader *object;
     HRSRC rsrc;
@@ -380,18 +381,30 @@ HRESULT WINAPI D3DX11CreateAsyncResourceLoaderW(HMODULE module, const WCHAR *res
 
     if (!(rsrc = FindResourceW(module, resource, (const WCHAR *)RT_RCDATA)))
     {
-        ERR("Failed to find resource.\n");
+        WARN("Failed to find resource.\n");
         HeapFree(GetProcessHeap(), 0, object);
-        return D3DX11_ERR_INVALID_DATA;
+        return D3DX10_ERR_INVALID_DATA;
     }
 
-    object->ID3DX11DataLoader_iface.lpVtbl = &resourcedataloadervtbl;
+    object->ID3DX10DataLoader_iface.lpVtbl = &resourcedataloadervtbl;
     object->u.resource.module = module;
     object->u.resource.rsrc = rsrc;
     object->data = NULL;
     object->size = 0;
 
-    *loader = &object->ID3DX11DataLoader_iface;
+    *loader = &object->ID3DX10DataLoader_iface;
 
     return S_OK;
 }
+
+HRESULT WINAPI D3DX10PreprocessShaderFromMemory(const char *data, SIZE_T data_size, const char *filename,
+        const D3D10_SHADER_MACRO *defines, ID3DInclude *include, ID3DX10ThreadPump *pump, ID3D10Blob **shader_text,
+        ID3D10Blob **errors, HRESULT *hresult)
+{
+    FIXME("data %s, data_size %lu, filename %s, defines %p, include %p, pump %p, shader_text %p, "
+            "errors %p, hresult %p stub!\n",
+            debugstr_an(data, data_size), data_size, debugstr_a(filename), defines, include, pump,
+            shader_text, errors, hresult);
+
+    return E_NOTIMPL;
+}
