@@ -82,7 +82,7 @@ HRESULT WINAPI DwmExtendFrameIntoClientArea(HWND hwnd, const MARGINS* margins)
 {
     FIXME("(%p, %p) stub\n", hwnd, margins);
 
-    return E_NOTIMPL;
+    return S_OK;
 }
 
 /**********************************************************************
@@ -269,6 +269,15 @@ HRESULT WINAPI DwmSetPresentParameters(HWND hwnd, DWM_PRESENT_PARAMETERS *params
     return S_OK;
 };
 
+/**********************************************************************
+ *           DwmSetIconicLivePreviewBitmap         (DWMAPI.@)
+ */
+HRESULT WINAPI DwmSetIconicLivePreviewBitmap(HWND hwnd, HBITMAP hbmp, POINT *pos, DWORD flags)
+{
+    FIXME("(%p %p %p %x) stub\n", hwnd, hbmp, pos, flags);
+    return S_OK;
+};
+
 /**********************************************************************
  *           DwmSetIconicThumbnail         (DWMAPI.@)
  */
@@ -277,3 +286,12 @@ HRESULT WINAPI DwmSetIconicThumbnail(HWND hwnd, HBITMAP hbmp, DWORD flags)
     FIXME("(%p %p %x) stub\n", hwnd, hbmp, flags);
     return S_OK;
 };
+
+/**********************************************************************
+ *           DwmpGetColorizationParameters         (DWMAPI.@)
+ */
+HRESULT WINAPI DwmpGetColorizationParameters(void *params)
+{
+    FIXME("(%p) stub\n", params);
+    return E_NOTIMPL;
+}
