@@ -1079,6 +1079,14 @@ __int64 CDECL MSVCRT__atoi64_l(const char *str, MSVCRT__locale_t locale)
     return MSVCRT_strtoi64_l(str, NULL, 10, locale);
 }
 
+/******************************************************************
+ *      _atoi64 (MSVCRT.@)
+ */
+__int64 CDECL MSVCRT__atoi64(const char *str)
+{
+    return MSVCRT_strtoi64_l(str, NULL, 10, NULL);
+}
+
 /******************************************************************
  *      _atol_l (MSVCRT.@)
  */
@@ -1096,6 +1104,14 @@ MSVCRT_long CDECL MSVCRT__atol_l(const char *str, MSVCRT__locale_t locale)
     return ret;
 }
 
+/******************************************************************
+ *      atol (MSVCRT.@)
+ */
+MSVCRT_long CDECL MSVCRT_atol(const char *str)
+{
+    return MSVCRT__atol_l(str, NULL);
+}
+
 #if _MSVCR_VER>=120
 
 /******************************************************************
@@ -2028,3 +2044,19 @@ int __cdecl MSVCRT__memicmp(const char *s1, const char *s2, MSVCRT_size_t len)
 {
     return MSVCRT__memicmp_l(s1, s2, len, NULL);
 }
+
+/*********************************************************************
+ *                  strcspn   (MSVCRT.@)
+ */
+MSVCRT_size_t __cdecl MSVCRT_strcspn(const char *str, const char *reject)
+{
+    return strcspn( str, reject );
+}
+
+/*********************************************************************
+ *                  strpbrk   (MSVCRT.@)
+ */
+char* __cdecl MSVCRT_strpbrk(const char *str, const char *accept)
+{
+    return strpbrk(str, accept);
+}
