@@ -267,6 +267,14 @@ MSVCRT_wchar_t* CDECL MSVCRT__wcsupr_l( MSVCRT_wchar_t *str, MSVCRT__locale_t lo
     return str;
 }
 
+/******************************************************************
+ *              _wcsupr (MSVCRT.@)
+ */
+MSVCRT_wchar_t* CDECL MSVCRT__wcsupr( MSVCRT_wchar_t *str )
+{
+    return MSVCRT__wcsupr_l(str, NULL);
+}
+
 /******************************************************************
  *		_wcslwr_s_l (MSVCRT.@)
  */
@@ -2216,6 +2224,15 @@ MSVCRT_long CDECL MSVCRT__wcstol_l(const MSVCRT_wchar_t *s,
     return ret;
 }
 
+/*********************************************************************
+ *  wcstol (MSVCRT.@)
+ */
+MSVCRT_long CDECL MSVCRT_wcstol(const MSVCRT_wchar_t *s,
+        MSVCRT_wchar_t **end, int base)
+{
+    return MSVCRT__wcstol_l(s, end, base, NULL);
+}
+
 /*********************************************************************
  *  _wtoi_l (MSVCRT.@)
  */
@@ -2431,6 +2448,14 @@ MSVCRT_wchar_t* CDECL MSVCRT_wcschr(const MSVCRT_wchar_t *str, MSVCRT_wchar_t ch
     return strchrW(str, ch);
 }
 
+/*********************************************************************
+ *              wcsrchr (MSVCRT.@)
+ */
+MSVCRT_wchar_t* CDECL MSVCRT_wcsrchr(const MSVCRT_wchar_t *str, MSVCRT_wchar_t ch)
+{
+    return strrchrW(str, ch);
+}
+
 /***********************************************************************
  *              wcslen (MSVCRT.@)
  */
@@ -2547,3 +2572,11 @@ MSVCRT_size_t CDECL MSVCRT_wcsxfrm(MSVCRT_wchar_t *dest,
 {
     return MSVCRT__wcsxfrm_l(dest, src, len, NULL);
 }
+
+/*********************************************************************
+ *              wcscmp (MSVCRT.@)
+ */
+int CDECL MSVCRT_wcscmp(const MSVCRT_wchar_t *str1, const MSVCRT_wchar_t *str2)
+{
+    return strcmpW(str1, str2);
+}
