@@ -17,6 +17,8 @@
  *
  */
 
+#include <math.h>
+
 #define COBJMACROS
 #define CONST_VTABLE
 
@@ -27,13 +29,18 @@
 #include "wine/test.h"
 
 #include "tmarshal.h"
-#include "tmarshal_dispids.h"
 
 static HRESULT (WINAPI *pVarAdd)(LPVARIANT,LPVARIANT,LPVARIANT);
 
 
 #define ok_ole_success(hr, func) ok(hr == S_OK, #func " failed with error 0x%08x\n", hr)
 
+#ifdef __i386__
+static const int tmarshal_todo = 0;
+#else
+static const int tmarshal_todo = 1;
+#endif
+
 /* ULL suffix is not portable */
 #define ULL_CONST(dw1, dw2) ((((ULONGLONG)dw1) << 32) | (ULONGLONG)dw2)
 
@@ -719,7 +726,7 @@ static HRESULT WINAPI Widget_VarArg(
 static BOOL mystruct_uint_ordered(MYSTRUCT *mystruct)
 {
     int i;
-    for (i = 0; i < sizeof(mystruct->uarr)/sizeof(mystruct->uarr[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(mystruct->uarr); i++)
         if (mystruct->uarr[i] != i)
             return FALSE;
     return TRUE;
@@ -917,6 +924,44 @@ static HRESULT WINAPI Widget_Coclass(
     return S_OK;
 }
 
+static HRESULT WINAPI Widget_basetypes_in(IWidget *iface, signed char c, short s, int i, hyper h,
+        unsigned char uc, unsigned short us, unsigned int ui, MIDL_uhyper uh,
+        float f, double d, STATE st)
+{
+    ok(c == 5, "Got char %d.\n", c);
+    ok(s == -123, "Got short %d.\n", s);
+    ok(i == -100000, "Got int %d.\n", i);
+    ok(h == (LONGLONG)-100000 * 1000000, "Got hyper %s.\n", wine_dbgstr_longlong(h));
+    ok(uc == 0, "Got unsigned char %u.\n", uc);
+    ok(us == 456, "Got unsigned short %u.\n", us);
+    ok(ui == 0xdeadbeef, "Got unsigned int %i.\n", ui);
+    ok(uh == (ULONGLONG)1234567890 * 9876543210, "Got unsigned hyper %s.\n", wine_dbgstr_longlong(uh));
+    ok(f == (float)M_PI, "Got float %f.\n", f);
+    ok(d == M_E, "Got double %f.\n", d);
+    ok(st == STATE_WIDGETIFIED, "Got state %u.\n", st);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI Widget_basetypes_out(IWidget *iface, signed char *c, short *s, int *i, hyper *h,
+        unsigned char *uc, unsigned short *us, unsigned int *ui, MIDL_uhyper *uh,
+        float *f, double *d, STATE *st)
+{
+    *c = 10;
+    *s = -321;
+    *i = -200000;
+    *h = (LONGLONG)-200000 * 1000000;
+    *uc = 254;
+    *us = 256;
+    *ui = 0xf00dfade;
+    *uh = (((ULONGLONG)0xabcdef01) << 32) | (ULONGLONG)0x23456789;
+    *f = M_LN2;
+    *d = M_LN10;
+    *st = STATE_UNWIDGETIFIED;
+
+    return S_OK;
+}
+
 static const struct IWidgetVtbl Widget_VTable =
 {
     Widget_QueryInterface,
@@ -958,6 +1003,8 @@ static const struct IWidgetVtbl Widget_VTable =
     Widget_VarArg_Run,
     Widget_VarArg_Ref_Run,
     Widget_Coclass,
+    Widget_basetypes_in,
+    Widget_basetypes_out,
 };
 
 static HRESULT WINAPI StaticWidget_QueryInterface(IStaticWidget *iface, REFIID riid, void **ppvObject)
@@ -1271,6 +1318,90 @@ static ITypeInfo *NonOleAutomation_GetTypeInfo(void)
     return NULL;
 }
 
+static void test_marshal_basetypes(IWidget *widget, IDispatch *disp)
+{
+    VARIANTARG arg[11];
+    DISPPARAMS dispparams = {arg, NULL, ARRAY_SIZE(arg), 0};
+    HRESULT hr;
+
+    signed char c;
+    short s;
+    int i;
+    hyper h;
+    unsigned char uc;
+    unsigned short us;
+    unsigned int ui;
+    MIDL_uhyper uh;
+    float f;
+    double d;
+    STATE st;
+
+    V_VT(&arg[10]) = VT_I1;     V_I1(&arg[10]) = 5;
+    V_VT(&arg[9])  = VT_I2;     V_I2(&arg[9])  = -123;
+    V_VT(&arg[8])  = VT_I4;     V_I4(&arg[8])  = -100000;
+    V_VT(&arg[7])  = VT_I8;     V_I8(&arg[7])  = (LONGLONG)-100000 * 1000000;
+    V_VT(&arg[6])  = VT_UI1;    V_UI1(&arg[6]) = 0;
+    V_VT(&arg[5])  = VT_UI2;    V_UI2(&arg[5]) = 456;
+    V_VT(&arg[4])  = VT_UI4;    V_UI4(&arg[4]) = 0xdeadbeef;
+    V_VT(&arg[3])  = VT_UI8;    V_UI8(&arg[3]) = (ULONGLONG)1234567890 * 9876543210;
+    V_VT(&arg[2])  = VT_R4;     V_R4(&arg[2])  = M_PI;
+    V_VT(&arg[1])  = VT_R8;     V_R8(&arg[1])  = M_E;
+    V_VT(&arg[0])  = VT_I4;     V_I4(&arg[0])  = STATE_WIDGETIFIED;
+    hr = IDispatch_Invoke(disp, DISPID_TM_BASETYPES_IN, &IID_NULL, LOCALE_NEUTRAL,
+            DISPATCH_METHOD, &dispparams, NULL, NULL, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IWidget_basetypes_in(widget, 5, -123, -100000, (LONGLONG)-100000 * 1000000, 0, 456,
+            0xdeadbeef, (ULONGLONG)1234567890 * 9876543210, M_PI, M_E, STATE_WIDGETIFIED);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    c = s = i = h = uc = us = ui = uh = f = d = st = 0;
+
+    V_VT(&arg[10]) = VT_BYREF|VT_I1;  V_I1REF(&arg[10]) = &c;
+    V_VT(&arg[9])  = VT_BYREF|VT_I2;  V_I2REF(&arg[9])  = &s;
+    V_VT(&arg[8])  = VT_BYREF|VT_I4;  V_I4REF(&arg[8])  = &i;
+    V_VT(&arg[7])  = VT_BYREF|VT_I8;  V_I8REF(&arg[7])  = &h;
+    V_VT(&arg[6])  = VT_BYREF|VT_UI1; V_UI1REF(&arg[6]) = &uc;
+    V_VT(&arg[5])  = VT_BYREF|VT_UI2; V_UI2REF(&arg[5]) = &us;
+    V_VT(&arg[4])  = VT_BYREF|VT_UI4; V_UI4REF(&arg[4]) = &ui;
+    V_VT(&arg[3])  = VT_BYREF|VT_UI8; V_UI8REF(&arg[3]) = &uh;
+    V_VT(&arg[2])  = VT_BYREF|VT_R4;  V_R4REF(&arg[2])  = &f;
+    V_VT(&arg[1])  = VT_BYREF|VT_R8;  V_R8REF(&arg[1])  = &d;
+    V_VT(&arg[0])  = VT_BYREF|VT_I4;  V_I4REF(&arg[0])  = (int *)&st;
+    hr = IDispatch_Invoke(disp, DISPID_TM_BASETYPES_OUT, &IID_NULL, LOCALE_NEUTRAL,
+            DISPATCH_METHOD, &dispparams, NULL, NULL, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(c == 10, "Got char %d.\n", c);
+    ok(s == -321, "Got short %d.\n", s);
+    ok(i == -200000, "Got int %d.\n", i);
+    ok(h == (LONGLONG)-200000 * 1000000L, "Got hyper %s.\n", wine_dbgstr_longlong(h));
+    ok(uc == 254, "Got unsigned char %u.\n", uc);
+    ok(us == 256, "Got unsigned short %u.\n", us);
+    ok(ui == 0xf00dfade, "Got unsigned int %i.\n", ui);
+    ok(uh == ((((ULONGLONG)0xabcdef01) << 32) | (ULONGLONG)0x23456789),
+            "Got unsigned hyper %s.\n", wine_dbgstr_longlong(uh));
+    ok(f == (float)M_LN2, "Got float %f.\n", f);
+    ok(d == M_LN10, "Got double %f.\n", d);
+    ok(st == STATE_UNWIDGETIFIED, "Got state %u.\n", st);
+
+    c = s = i = h = uc = us = ui = uh = f = d = st = 0;
+
+    hr = IWidget_basetypes_out(widget, &c, &s, &i, &h, &uc, &us, &ui, &uh, &f, &d, &st);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(c == 10, "Got char %d.\n", c);
+    ok(s == -321, "Got short %d.\n", s);
+    ok(i == -200000, "Got int %d.\n", i);
+    ok(h == (LONGLONG)-200000 * 1000000L, "Got hyper %s.\n", wine_dbgstr_longlong(h));
+    ok(uc == 254, "Got unsigned char %u.\n", uc);
+    ok(us == 256, "Got unsigned short %u.\n", us);
+    ok(ui == 0xf00dfade, "Got unsigned int %i.\n", ui);
+    ok(uh == ((((ULONGLONG)0xabcdef01) << 32) | (ULONGLONG)0x23456789),
+            "Got unsigned hyper %s.\n", wine_dbgstr_longlong(uh));
+    ok(f == (float)M_LN2, "Got float %f.\n", f);
+    ok(d == M_LN10, "Got double %f.\n", d);
+    ok(st == STATE_UNWIDGETIFIED, "Got state %u.\n", st);
+}
+
 static void test_typelibmarshal(void)
 {
     static const WCHAR szCat[] = { 'C','a','t',0 };
@@ -1305,6 +1436,7 @@ static void test_typelibmarshal(void)
 
     IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
     hr = CoUnmarshalInterface(pStream, &IID_IKindaEnumWidget, (void **)&pKEW);
+    todo_wine_if(tmarshal_todo)
     ok_ole_success(hr, CoUnmarshalInterface);
     IStream_Release(pStream);
 
@@ -1331,6 +1463,7 @@ static void test_typelibmarshal(void)
     VariantInit(&varresult);
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_NAME, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_PROPERTYPUT, &dispparams, &varresult, &excepinfo, NULL);
     ok_ole_success(hr, IDispatch_Invoke);
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1350,6 +1483,7 @@ static void test_typelibmarshal(void)
     VariantInit(&varresult);
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_NAME, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_PROPERTYGET, &dispparams, &varresult, &excepinfo, NULL);
     ok_ole_success(hr, IDispatch_Invoke);
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1481,6 +1615,7 @@ static void test_typelibmarshal(void)
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_CLONEDISPATCH, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_PROPERTYGET, &dispparams, &varresult, &excepinfo, NULL);
     ok_ole_success(hr, IDispatch_Invoke);
 
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1498,6 +1633,7 @@ static void test_typelibmarshal(void)
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_CLONECOCLASS, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_PROPERTYGET, &dispparams, &varresult, &excepinfo, NULL);
     ok_ole_success(hr, IDispatch_Invoke);
 
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
        "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
        excepinfo.wCode, excepinfo.scode);
@@ -1514,6 +1650,7 @@ static void test_typelibmarshal(void)
     VariantInit(&varresult);
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_COCLASS, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_METHOD, &dispparams, &varresult, &excepinfo, NULL);
     todo_wine ok_ole_success(hr, IDispatch_Invoke);
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1533,6 +1670,7 @@ static void test_typelibmarshal(void)
     hr = IDispatch_Invoke(pDispatch, DISPID_VALUE, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_PROPERTYGET, &dispparams, &varresult, &excepinfo, NULL);
     ok_ole_success(hr, IDispatch_Invoke);
 
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == S_OK,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1588,9 +1726,9 @@ static void test_typelibmarshal(void)
     V_I4(&vararg[1]) = 2;
     hr = IWidget_VariantCArray(pWidget, 2, vararg);
     ok_ole_success(hr, IWidget_VariantCArray);
-    todo_wine
+    todo_wine_if(!tmarshal_todo)
     ok(V_VT(&vararg[0]) == VT_I4 && V_I4(&vararg[0]) == 2, "vararg[0] = %d[%d]\n", V_VT(&vararg[0]), V_I4(&vararg[0]));
-    todo_wine
+    todo_wine_if(!tmarshal_todo)
     ok(V_VT(&vararg[1]) == VT_I4 && V_I4(&vararg[1]) == 3, "vararg[1] = %d[%d]\n", V_VT(&vararg[1]), V_I4(&vararg[1]));
 
     /* call VarArg */
@@ -1661,6 +1799,7 @@ static void test_typelibmarshal(void)
     VariantInit(&varresult);
     hr = IDispatch_Invoke(pDispatch, DISPID_TM_ERROR, &IID_NULL, LOCALE_NEUTRAL, DISPATCH_METHOD, &dispparams, NULL, &excepinfo, NULL);
     ok(hr == DISP_E_EXCEPTION, "IDispatch_Invoke should have returned DISP_E_EXCEPTION instead of 0x%08x\n", hr);
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == E_NOTIMPL,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1687,6 +1826,7 @@ static void test_typelibmarshal(void)
     hr = ITypeInfo_Invoke(pTypeInfo, &NonOleAutomation, DISPID_NOA_ERROR, DISPATCH_METHOD, &dispparams, &varresult, &excepinfo, NULL);
     ok(hr == DISP_E_EXCEPTION, "ITypeInfo_Invoke should have returned DISP_E_EXCEPTION instead of 0x%08x\n", hr);
     ok(V_VT(&varresult) == VT_EMPTY, "V_VT(&varresult) should be VT_EMPTY instead of %d\n", V_VT(&varresult));
+    todo_wine_if(tmarshal_todo)
     ok(excepinfo.wCode == 0x0 && excepinfo.scode == E_NOTIMPL,
         "EXCEPINFO differs from expected: wCode = 0x%x, scode = 0x%08x\n",
         excepinfo.wCode, excepinfo.scode);
@@ -1872,6 +2012,8 @@ static void test_typelibmarshal(void)
     ok(V_I4(&varresult) == DISPID_TM_NEG_RESTRICTED, "got %x\n", V_I4(&varresult));
     VariantClear(&varresult);
 
+    test_marshal_basetypes(pWidget, pDispatch);
+
     IDispatch_Release(pDispatch);
     IWidget_Release(pWidget);
 
@@ -1998,6 +2140,7 @@ static void test_external_connection(void)
 
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoUnmarshalInterface(stream, &IID_ItestDual, (void**)&iface);
+    todo_wine_if(tmarshal_todo)
     ok(hres == S_OK, "CoUnmarshalInterface failed: %08x\n", hres);
     ok(external_connections == 1, "external_connections = %d\n", external_connections);
 
@@ -2006,6 +2149,7 @@ static void test_external_connection(void)
 
     /* Creating a stub for new iface causes new external connection. */
     hres = ItestDual_QueryInterface(iface, &IID_ITestSecondDisp, (void**)&second);
+    todo_wine_if(tmarshal_todo)
     ok(hres == S_OK, "Could not get ITestSecondDisp iface: %08x\n", hres);
     todo_wine
     ok(external_connections == 2, "external_connections = %d\n", external_connections);
@@ -2017,6 +2161,7 @@ static void test_external_connection(void)
 
     expect_last_release_closes = TRUE;
     ItestDual_Release(iface);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
 
     end_host_object(tid, thread);
@@ -2028,12 +2173,14 @@ static void test_external_connection(void)
     expect_last_release_closes = FALSE;
     hres = CoMarshalInterface(stream, &IID_ItestDual, (IUnknown*)&TestDual, MSHCTX_INPROC, NULL, MSHLFLAGS_NORMAL);
     ok(hres == S_OK, "CoMarshalInterface failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 1, "external_connections = %d\n", external_connections);
 
     expect_last_release_closes = TRUE;
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoReleaseMarshalData(stream);
     ok(hres == S_OK, "CoReleaseMarshalData failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
 
     /* Two separated marshal data are still one external connection. */
@@ -2044,21 +2191,25 @@ static void test_external_connection(void)
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoMarshalInterface(stream, &IID_ItestDual, (IUnknown*)&TestDual, MSHCTX_INPROC, NULL, MSHLFLAGS_NORMAL);
     ok(hres == S_OK, "CoMarshalInterface failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 1, "external_connections = %d\n", external_connections);
 
     hres = CoMarshalInterface(stream2, &IID_ItestDual, (IUnknown*)&TestDual, MSHCTX_INPROC, NULL, MSHLFLAGS_NORMAL);
     ok(hres == S_OK, "CoMarshalInterface failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 1, "external_connections = %d\n", external_connections);
 
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoReleaseMarshalData(stream);
     ok(hres == S_OK, "CoReleaseMarshalData failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 1, "external_connections = %d\n", external_connections);
 
     expect_last_release_closes = TRUE;
     IStream_Seek(stream2, zero, STREAM_SEEK_SET, NULL);
     hres = CoReleaseMarshalData(stream2);
     ok(hres == S_OK, "CoReleaseMarshalData failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
 
     IStream_Release(stream);
@@ -2070,17 +2221,20 @@ static void test_external_connection(void)
 
     hres = CoMarshalInterface(stream, &IID_ItestDual, (IUnknown*)&TestDual, MSHCTX_INPROC, NULL, MSHLFLAGS_TABLEWEAK);
     ok(hres == S_OK, "CoMarshalInterface failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
 
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoUnmarshalInterface(stream, &IID_ItestDual, (void**)&iface);
     ok(hres == S_OK, "CoUnmarshalInterface failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
     ItestDual_Release(iface);
 
     IStream_Seek(stream, zero, STREAM_SEEK_SET, NULL);
     hres = CoReleaseMarshalData(stream);
     ok(hres == S_OK, "CoReleaseMarshalData failed: %08x\n", hres);
+    todo_wine_if(tmarshal_todo)
     ok(external_connections == 0, "external_connections = %d\n", external_connections);
 
     IStream_Release(stream);
