@@ -207,7 +207,7 @@ static void create_texture_view(struct wined3d_gl_view *view, GLenum view_target
             level_idx, desc->u.texture.level_count, layer_idx, layer_count));
     checkGLcall("create texture view");
 
-    if (is_stencil_view_format(&view_format_gl->f))
+    if (is_stencil_view_format(view_format))
     {
         static const GLint swizzle[] = {GL_ZERO, GL_RED, GL_ZERO, GL_ZERO};
 
@@ -226,6 +226,18 @@ static void create_texture_view(struct wined3d_gl_view *view, GLenum view_target
         context_invalidate_compute_state(context, STATE_COMPUTE_SHADER_RESOURCE_BINDING);
         context_invalidate_state(context, STATE_GRAPHICS_SHADER_RESOURCE_BINDING);
     }
+    else if (!is_identity_fixup(view_format->color_fixup) && can_use_texture_swizzle(gl_info, view_format))
+    {
+        GLint swizzle[4];
+
+        context_bind_texture(context, view->target, view->name);
+        wined3d_gl_texture_swizzle_from_color_fixup(swizzle, view_format->color_fixup);
+        gl_info->gl_ops.gl.p_glTexParameteriv(view->target, GL_TEXTURE_SWIZZLE_RGBA, swizzle);
+        checkGLcall("set format swizzle");
+
+        context_invalidate_compute_state(context, STATE_COMPUTE_SHADER_RESOURCE_BINDING);
+        context_invalidate_state(context, STATE_GRAPHICS_SHADER_RESOURCE_BINDING);
+    }
 
     context_release(context);
 }
@@ -611,8 +623,8 @@ HRESULT CDECL wined3d_rendertarget_view_create(const struct wined3d_view_desc *d
     struct wined3d_rendertarget_view *object;
     HRESULT hr;
 
-    TRACE("desc %p, resource %p, parent %p, parent_ops %p, view %p.\n",
-            desc, resource, parent, parent_ops, view);
+    TRACE("desc %s, resource %p, parent %p, parent_ops %p, view %p.\n",
+            wined3d_debug_view_desc(desc, resource), resource, parent, parent_ops, view);
 
     if (!(object = heap_alloc_zero(sizeof(*object))))
         return E_OUTOFMEMORY;
@@ -790,8 +802,8 @@ HRESULT CDECL wined3d_shader_resource_view_create(const struct wined3d_view_desc
     struct wined3d_shader_resource_view *object;
     HRESULT hr;
 
-    TRACE("desc %p, resource %p, parent %p, parent_ops %p, view %p.\n",
-            desc, resource, parent, parent_ops, view);
+    TRACE("desc %s, resource %p, parent %p, parent_ops %p, view %p.\n",
+            wined3d_debug_view_desc(desc, resource), resource, parent, parent_ops, view);
 
     if (!(object = heap_alloc_zero(sizeof(*object))))
         return E_OUTOFMEMORY;
@@ -1011,8 +1023,8 @@ void wined3d_unordered_access_view_clear_uint(struct wined3d_unordered_access_vi
 {
     const struct wined3d_gl_info *gl_info = context->gl_info;
     const struct wined3d_format_gl *format;
+    struct wined3d_buffer_gl *buffer_gl;
     struct wined3d_resource *resource;
-    struct wined3d_buffer *buffer;
     unsigned int offset, size;
 
     resource = view->resource;
@@ -1037,13 +1049,13 @@ void wined3d_unordered_access_view_clear_uint(struct wined3d_unordered_access_vi
         return;
     }
 
-    buffer = buffer_from_resource(resource);
-    wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_BUFFER);
+    buffer_gl = wined3d_buffer_gl(buffer_from_resource(resource));
+    wined3d_buffer_load_location(&buffer_gl->b, context, WINED3D_LOCATION_BUFFER);
     wined3d_unordered_access_view_invalidate_location(view, ~WINED3D_LOCATION_BUFFER);
 
-    get_buffer_view_range(buffer, &view->desc, &format->f, &offset, &size);
-    context_bind_bo(context, buffer->buffer_type_hint, buffer->buffer_object);
-    GL_EXTCALL(glClearBufferSubData(buffer->buffer_type_hint, format->internal,
+    get_buffer_view_range(&buffer_gl->b, &view->desc, &format->f, &offset, &size);
+    context_bind_bo(context, buffer_gl->buffer_type_hint, buffer_gl->b.buffer_object);
+    GL_EXTCALL(glClearBufferSubData(buffer_gl->buffer_type_hint, format->internal,
             offset, size, format->format, format->type, clear_value));
     checkGLcall("clear unordered access view");
 }
@@ -1080,7 +1092,7 @@ void wined3d_unordered_access_view_copy_counter(struct wined3d_unordered_access_
     src.buffer_object = view->counter_bo;
     src.addr = NULL;
 
-    context_copy_bo_address(context, &dst, buffer->buffer_type_hint,
+    context_copy_bo_address(context, &dst, wined3d_buffer_gl(buffer)->buffer_type_hint,
             &src, GL_ATOMIC_COUNTER_BUFFER, sizeof(GLuint));
 
     wined3d_buffer_invalidate_location(buffer, ~dst_location);
@@ -1158,8 +1170,8 @@ HRESULT CDECL wined3d_unordered_access_view_create(const struct wined3d_view_des
     struct wined3d_unordered_access_view *object;
     HRESULT hr;
 
-    TRACE("desc %p, resource %p, parent %p, parent_ops %p, view %p.\n",
-            desc, resource, parent, parent_ops, view);
+    TRACE("desc %s, resource %p, parent %p, parent_ops %p, view %p.\n",
+            wined3d_debug_view_desc(desc, resource), resource, parent, parent_ops, view);
 
     if (!(object = heap_alloc_zero(sizeof(*object))))
         return E_OUTOFMEMORY;
