@@ -89,8 +89,6 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD reason, void *pReserved)
 
     switch (reason)
     {
-    case DLL_WINE_PREATTACH:
-        return FALSE;  /* prefer native version */
     case DLL_PROCESS_ATTACH:
         instance = hinstDLL;
         DisableThreadLibraryCalls( hinstDLL );
@@ -337,7 +335,35 @@ static HRESULT WINAPI XA2SRC_SetChannelVolumes(IXAudio2SourceVoice *iface,
         UINT32 Channels, const float *pVolumes, UINT32 OperationSet)
 {
     XA2SourceImpl *This = impl_from_IXAudio2SourceVoice(iface);
+    ALfloat al_gain;
+    UINT32 i;
+    BOOL same_volumes_given = TRUE;
+
     TRACE("%p, %u, %p, 0x%x\n", This, Channels, pVolumes, OperationSet);
+
+#if XAUDIO2_VER > 7
+    if(Channels != This->fmt->nChannels || !pVolumes)
+        return COMPAT_E_INVALID_CALL;
+#endif
+
+    al_gain = *pVolumes;
+
+    /* check whether all volumes are the same */
+    for(i = 1; i < Channels; ++i){
+        if(al_gain != *(pVolumes + i)){
+            same_volumes_given = FALSE;
+            break;
+        }
+    }
+    if(!same_volumes_given){
+        WARN("Different volumes for channels unsupported, setting the highest volume.\n");
+        for(; i < Channels; ++i)
+            al_gain = max(al_gain, *(pVolumes + i));
+    }
+
+    palcSetThreadContext(This->xa2->al_ctx);
+    alSourcef(This->al_src, AL_GAIN, al_gain);
+
     return S_OK;
 }
 
@@ -1236,7 +1262,9 @@ static HRESULT WINAPI IXAudio2Impl_QueryInterface(IXAudio2 *iface, REFIID riid,
     TRACE("(%p)->(%s, %p)\n", This, debugstr_guid(riid), ppvObject);
 
     if(IsEqualGUID(riid, &IID_IUnknown) ||
+#if XAUDIO2_VER == 8
             IsEqualGUID(riid, &IID_IXAudio28) ||
+#endif
             IsEqualGUID(riid, &IID_IXAudio2))
         *ppvObject = &This->IXAudio2_iface;
     else if(IsEqualGUID(riid, &IID_IXAudio27)){
@@ -2093,7 +2121,8 @@ static HRESULT WINAPI XAudio2CF_CreateInstance(IClassFactory *iface, IUnknown *p
 
 static HRESULT WINAPI XAudio2CF_LockServer(IClassFactory *iface, BOOL dolock)
 {
-    FIXME("(static)->(%d): stub!\n", dolock);
+    struct xaudio2_cf *This = impl_from_IClassFactory(iface);
+    FIXME("(%p)->(%d): stub!\n", This, dolock);
     return S_OK;
 }
 
@@ -2106,18 +2135,20 @@ static const IClassFactoryVtbl XAudio2CF_Vtbl =
     XAudio2CF_LockServer
 };
 
-static IClassFactory *make_xaudio2_factory(void)
+static HRESULT make_xaudio2_factory(REFIID riid, void **ppv)
 {
+    HRESULT hr;
     struct xaudio2_cf *ret = HeapAlloc(GetProcessHeap(), 0, sizeof(struct xaudio2_cf));
     ret->IClassFactory_iface.lpVtbl = &XAudio2CF_Vtbl;
     ret->ref = 0;
-    return &ret->IClassFactory_iface;
+    hr = IClassFactory_QueryInterface(&ret->IClassFactory_iface, riid, ppv);
+    if(FAILED(hr))
+        HeapFree(GetProcessHeap(), 0, ret);
+    return hr;
 }
 
 HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
 {
-    IClassFactory *factory = NULL;
-
     TRACE("(%s, %s, %p)\n", debugstr_guid(rclsid), debugstr_guid(riid), ppv);
 
     if(IsEqualGUID(rclsid, &CLSID_XAudio20) ||
@@ -2127,33 +2158,30 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
             IsEqualGUID(rclsid, &CLSID_XAudio24) ||
             IsEqualGUID(rclsid, &CLSID_XAudio25) ||
             IsEqualGUID(rclsid, &CLSID_XAudio26) ||
-            IsEqualGUID(rclsid, &CLSID_XAudio27)){
-        factory = make_xaudio2_factory();
+            IsEqualGUID(rclsid, &CLSID_XAudio27))
+        return make_xaudio2_factory(riid, ppv);
 
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter20) ||
+    if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter20) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter21) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter22) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter23) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter24) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter25) ||
                 IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter26) ||
-                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter27)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter27);
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter27))
+        return make_xapo_factory(&CLSID_AudioVolumeMeter27, riid, ppv);
 
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb20) ||
+    if(IsEqualGUID(rclsid, &CLSID_AudioReverb20) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb21) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb22) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb23) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb24) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb25) ||
                 IsEqualGUID(rclsid, &CLSID_AudioReverb26) ||
-                IsEqualGUID(rclsid, &CLSID_AudioReverb27)){
-        factory = make_xapo_factory(&CLSID_FXReverb);
-    }
+                IsEqualGUID(rclsid, &CLSID_AudioReverb27))
+        return make_xapo_factory(&CLSID_FXReverb, riid, ppv);
 
-    if(!factory) return CLASS_E_CLASSNOTAVAILABLE;
-
-    return IClassFactory_QueryInterface(factory, riid, ppv);
+    return CLASS_E_CLASSNOTAVAILABLE;
 }
 
 HRESULT xaudio2_initialize(IXAudio2Impl *This, UINT32 flags, XAUDIO2_PROCESSOR proc)
@@ -2172,7 +2200,9 @@ HRESULT WINAPI XAudio2Create(IXAudio2 **ppxa2, UINT32 flags, XAUDIO2_PROCESSOR p
 
     TRACE("%p 0x%x 0x%x\n", ppxa2, flags, proc);
 
-    cf = make_xaudio2_factory();
+    hr = make_xaudio2_factory(&IID_IClassFactory, (void**)&cf);
+    if(FAILED(hr))
+        return hr;
 
     hr = IClassFactory_CreateInstance(cf, NULL, &IID_IXAudio2, (void**)&xa2);
     IClassFactory_Release(cf);
